'use strict';

const abacusRequest = require('abacus-request');
const generator = require('generate-password');
const httpStatus = require('http-status-codes');

const adminToken = require('../auth/admin-token.js');
const config = require('../config.js');

const retry = require('abacus-retry');
const throttle = require('abacus-throttle');
const breaker = require('abacus-breaker');

const request = throttle(retry(breaker(abacusRequest)));

const debug = require('abacus-debug')('abacus-broker');
const edebug = require('abacus-debug')('e-abacus-broker');

const createClient = (clientId, resourceId, cb) => {
  const scopes = ['abacus.usage.' + resourceId + '.read',
    'abacus.usage.' + resourceId + '.write'];
  const secret = generator.generate({
    length: 10,
    numbers: true
  });

  debug('Creating UAA client with id %s ', clientId);
  request.post(config.uris().auth_server + '/oauth/clients', {
    headers: adminToken.authHeader(),
    body: {
      scope : scopes,
      client_id : clientId,
      client_secret : secret,
      resource_ids : [ ],
      authorized_grant_types : [ 'client_credentials' ],
      authorities : scopes,
      autoapprove : true
    }
  }, (err, res) => {
    if(err) {
      edebug('Could not create UAA client %s due to %o', clientId, err);
      cb(httpStatus.INTERNAL_SERVER_ERROR, {});
    } else if (res.statusCode !== httpStatus.CREATED) {
      edebug('Could not create UAA client %s due to %s, %o',
        clientId, res.statusCode, res.body);
      cb(res.statusCode, {});
    } else {
      debug('Successfully created UAA client %s', clientId);
      cb(res.statusCode, {
        credentials: {
          client_id: clientId,
          client_secret: secret,
          collector_url: config.uris().collector + config.usageCollectorPath
        }
      });
    }
  });
};

const deleteClient = (clientId, cb) => {
  debug('Deleting UAA client with id %s', clientId);
  request.delete(config.uris().auth_server + '/oauth/clients/' + clientId, {
    headers: adminToken.authHeader()
  }, (err, res) => {
    if(err) {
      edebug('Could not delete UAA client %s due to %o', clientId, err);
      cb(httpStatus.INTERNAL_SERVER_ERROR);
    } else if (res.statusCode === httpStatus.NOT_FOUND) {
      debug('UAA client %s not found', clientId);
      cb(httpStatus.OK);
    } else if (res.statusCode === httpStatus.OK) {
      debug('Successfully deleted UAA client %s', clientId);
      cb(res.statusCode);
    } else {
      edebug('Could not delete UAA client %s due to %s, %o',
        clientId, res.statusCode, res.body);
      cb(res.statusCode);
    }
  });
};

module.exports.createClient = createClient;
module.exports.deleteClient = deleteClient;
